#! /usr/bin/env python

import sys
from os import utime
from os.path import getmtime
from getopt import getopt, GetoptError

import eyeD3
from eyeD3.frames import id3EncodingToString, \
    LATIN1_ENCODING, UTF_16_ENCODING, UTF_16BE_ENCODING, UTF_8_ENCODING
from m_lib.opstring import recode


def usage(msg):
    pname = sys.argv[0]
    sys.stderr.write("%s: %s\n" % (pname, msg))
    sys.stderr.write("Usage: %s [-f encoding|--from-encoding=encoding|-t encoding|--to-encoding=encoding] file.mp3...\n" % pname)
    sys.exit(1)

try:
    optlist, args = getopt(sys.argv[1:], "f:t:",
        ["from-encoding=", "to-encoding="]
    )
except GetoptError:
    usage("invalid option")


src_encoding = None
dst_encoding = None

for _opt, _arg in optlist:
    if _opt in ('-f', '--from-encoding'):
        src_encoding = _arg
    elif _opt in ('-t', '--to-encoding'):
        dst_encoding = _arg
try:
    del _opt, _arg
except NameError:
    pass

if not args:
    usage("too few arguments")

if not dst_encoding:
    from m_lib.defenc import default_encoding
    dst_encoding = default_encoding


def id3tag_set(frame, attr, src_encoding, dst_encoding):
    value = getattr(frame, attr)
    if frame.encoding == LATIN1_ENCODING:
        value = value.encode('latin1').decode(src_encoding).encode(dst_encoding).decode('latin1')
        setattr(frame, attr, value)
    elif frame.encoding in (UTF_16_ENCODING, UTF_16BE_ENCODING, UTF_8_ENCODING):
        #value = value.encode(dst_encoding).decode('latin1')
        #frame.encoding = LATIN1_ENCODING
        pass
    else:
        raise ValueError('Unknown encoding "%s"' % id3EncodingToString(frame.encoding))

for filename in args:
    mtime = getmtime(filename)
    for version in eyeD3.ID3_V2, eyeD3.ID3_V1:
        id3tag = eyeD3.Tag()
        if not id3tag.link(filename, version):
            continue

        if not src_encoding:
            src_encoding = id3EncodingToString(id3tag.encoding)

        for frame in id3tag.frames:
            if hasattr(frame, "text"):
                id3tag_set(frame, "text", src_encoding, dst_encoding)
            elif hasattr(frame, "comment"):
                id3tag_set(frame, "comment", src_encoding, dst_encoding)

        title_frame = id3tag.getTitle()
        if title_frame:
            try:
                print title_frame
            except UnicodeEncodeError:
                print title_frame.encode('latin1')
        else:
            print filename

        if id3tag.getVersion() == eyeD3.ID3_V2_2:
            version = eyeD3.ID3_DEFAULT_VERSION
        else:
            version = eyeD3.ID3_CURRENT_VERSION
        id3tag.update(version=version)
    utime(filename, (mtime, mtime))
